﻿using Syncfusion.UI.Xaml.Grid;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;
using Syncfusion.Data.Extensions;
using Syncfusion.Dynamic;

namespace SfDataGridDemo
{
    public interface IItemsSourceSelector
    {
        IEnumerable GetItemsSource(object record, object dataContext);
    }

    //Creating a CustomColumn for binding different ItemsSource to each row of GridComboBoxColumn..
    public class GridComboBoxColumnExt : GridComboBoxColumn
    {
        public IItemsSourceSelector ItemsSourceSelectorSelector
        {
            get { return (IItemsSourceSelector)GetValue(ItemsSourceSelectorSelectorProperty); }
            set { SetValue(ItemsSourceSelectorSelectorProperty, value); }
        }

        public static readonly DependencyProperty ItemsSourceSelectorSelectorProperty =
            GridDependencyProperty.Register("ItemsSourceSelectorSelector", typeof(IItemsSourceSelector), typeof(GridComboBoxColumnExt), new GridPropertyMetadata(null));

        public GridComboBoxColumnExt()
        {
            SetCellType("GridComboBoxExtRenderer");
        }       

        protected override void SetDisplayBindingConverter()
        {
            if (DisplayBinding is Binding && (DisplayBinding as Binding).Converter == null)
            {
                //Creating MultiBinding to pass Record and also Value (MappingName) to Converter.
                var multiBinding = new MultiBinding
                {
                    Converter = new DisplayMemberConverter(this),
                    ConverterParameter = this.DataGrid.DataContext
                };
                multiBinding.Bindings.Add(new Binding());
                multiBinding.Bindings.Add(new Binding() { Path = new PropertyPath(this.MappingName) });
                DisplayBinding = multiBinding;

            }
        }    
    }

    public class DisplayMemberConverter : IMultiValueConverter
    {
        private GridComboBoxColumnExt cachedColumn;

        public DisplayMemberConverter(GridComboBoxColumnExt column)
        {
            cachedColumn = column;
        }

        protected DynamicHelper dynamicHelper;
        public object Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
        {
            object record = values[0];
            var selectedValue = values[1];

            IEnumerable list = null;
            if (cachedColumn.ItemsSourceSelectorSelector != null)
                list = cachedColumn.ItemsSourceSelectorSelector.GetItemsSource(record, parameter);
            if (list == null)
                list = cachedColumn.ItemsSource;
            
            var displayMemberPath = string.Empty;
            var valueMemberPath = string.Empty;
            displayMemberPath = cachedColumn.DisplayMemberPath;
            valueMemberPath = cachedColumn.SelectedValuePath;

            if (selectedValue == null)
                return null;

            bool? isdynamic = null;

            PropertyDescriptorCollection pdc = null;
            bool? isdatarow = null;

            if (string.IsNullOrEmpty(valueMemberPath))
            {
                if (!string.IsNullOrEmpty(displayMemberPath))
                {
                    var type = selectedValue.GetType();
                    isdynamic = isdynamic ?? DynamicHelper.CheckIsDynamicObject(type);
                    if (isdynamic == true)
                    {
                        if (dynamicHelper == null)
                            dynamicHelper = new DynamicHelper();
                        return dynamicHelper.GetValue(selectedValue, displayMemberPath);
                    }
                    isdatarow = isdatarow ?? type == typeof(System.Data.DataRowView);
                    if (isdatarow == true)
                    {
                        return (selectedValue as System.Data.DataRowView)[displayMemberPath];
                    }
                    pdc = TypeDescriptor.GetProperties(type);

                    return pdc.GetValue(selectedValue, displayMemberPath);
                }
                return selectedValue;
            }
            
            if (list == null)
                return null;

            var enumerator = list.GetEnumerator();
            while (enumerator.MoveNext())
            {
                var type = enumerator.Current.GetType();

                isdynamic = isdynamic ?? DynamicHelper.CheckIsDynamicObject(type);
                if (isdynamic == true)
                {
                    if (dynamicHelper == null)
                        dynamicHelper = new DynamicHelper();
                    if (selectedValue.Equals(dynamicHelper.GetValue(enumerator.Current, valueMemberPath)))
                    {
                        if (!string.IsNullOrEmpty(displayMemberPath))
                            return dynamicHelper.GetValue(enumerator.Current, displayMemberPath);
                        return enumerator.Current;
                    }
                    continue;
                }
                    
                isdatarow = isdatarow ?? type == typeof(System.Data.DataRowView);
                if (isdatarow == true)
                {
                    var datarowview = (enumerator.Current as System.Data.DataRowView);
                    if (selectedValue.Equals(datarowview[valueMemberPath]))
                        return datarowview[displayMemberPath];
                    continue;
                }
                pdc = pdc ?? TypeDescriptor.GetProperties(type);

                if (selectedValue.Equals(pdc.GetValue(enumerator.Current, valueMemberPath)))
                {
                    if (!string.IsNullOrEmpty(displayMemberPath))
                        return pdc.GetValue(enumerator.Current, displayMemberPath);
                    return enumerator.Current;
                }
            }
            return null;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
        {
            return null;
        }
    }
}
